package acct

import (
	"github.com/sirupsen/logrus"

	"gitlab.com/oiacow/goheroes/backend/core"
)

type reqServerNuGetPersonas struct {
	// TXN=NuGetPersonas
	TXN string `fesl:"TXN"`
	// namespace=
	Namespace string `fesl:"namespace"`
}

type ansNuGetPersonas struct {
	Txn      string   `fesl:"TXN"`
	Personas []string `fesl:"personas"`
}

//NuGetPersonas handle acct.NuGetPersonas
//GetPersonas retrieve all the heroes for that owner
func (acct *Account) NuGetPersonas(event core.EClientCmd) {
	switch event.Client.GetClientType() {
	case clientTypeServer:
		acct.serverNuGetPersonas(event)
	default:
		acct.clientNuGetPersonas(event)
	}
}

func (acct *Account) clientNuGetPersonas(event core.EClientCmd) {
	hs, err := acct.DB.GetHeroesByPID(
		acct.DB.NewSession(),
		event.Client.PlayerData.PlayerID,
	)
	if err != nil {
		logrus.
			WithField("playerID", event.Client.PlayerData.PlayerID).
			Error("Cant fetch any heroes of player")
		return
	}

	ans := ansNuGetPersonas{Txn: acctNuGetPersonas, Personas: []string{}}

	for _, h := range hs {
		ans.Personas = append(ans.Personas, h.HeroName)
	}

	acct.answer(event.Client, event.Cmd.PayloadID, ans)
}

func (acct *Account) serverNuGetPersonas(event core.EClientCmd) {
	srv, err := acct.DB.GetServerByID(
		acct.DB.NewSession(),
		event.Client.PlayerData.ServerID,
	)
	if err != nil {
		logrus.WithError(err).Errorf("acct: Cant get server with ID '%d'", event.Client.PlayerData.ServerID)
		return
	}

	event.Client.PlayerData.ServerID = srv.ID
	event.Client.PlayerData.ServerUserName = srv.AccountUsername

	acct.answer(
		event.Client,
		event.Cmd.PayloadID,
		ansNuGetPersonas{
			Txn:      acctNuGetPersonas,
			Personas: []string{srv.AccountUsername},
		},
	)
}
