package acct

import (
	"github.com/sirupsen/logrus"

	"gitlab.com/oiacow/goheroes/backend/core"
)

type reqNuLoginPersona struct {
	// TXN=NuLoginPersona
	Txn  string `fesl:"TXN"`
	Name string `fesl:"name"` // Value specified in +soldierName
}

type ansNuLoginPersona struct {
	Txn       string `fesl:"TXN"`
	ProfileID int    `fesl:"profileId"`
	UserID    int    `fesl:"userId"`
	LobbyKey  string `fesl:"lkey"`
}

// NuLoginPersona handles acct.NuLoginPersona cmd
func (acct *Account) NuLoginPersona(event core.EClientCmd) {
	switch event.Client.GetClientType() {
	case clientTypeServer:
		acct.serverNuLoginPersona(event)
	default:
		acct.clientNuLoginPersona(event)
	}
}

// clientNuLoginPersona used when user selects hero from game client
func (acct *Account) clientNuLoginPersona(event core.EClientCmd) {
	heroName := event.Cmd.Msg["name"]

	h, err := acct.DB.GetHeroByName(acct.DB.NewSession(), heroName)
	if err != nil {
		logrus.WithError(err).Warnf("Cant get hero for name '%s'", heroName)
		return
	}

	event.Client.PlayerData.HeroID = h.ID
	event.Client.PlayerData.PlayerID = h.PlayerID
	event.Client.PlayerData.HeroName = h.HeroName

	acct.answer(
		event.Client,
		event.Cmd.PayloadID,
		ansNuLoginPersona{
			Txn:       acctNuLoginPersona,
			ProfileID: h.PlayerID,
			UserID:    h.ID,
			LobbyKey:  event.Client.PlayerData.LobbyKey,
		},
	)
}

// NuLoginPersonaServer - soldier login cmd
func (acct *Account) serverNuLoginPersona(event core.EClientCmd) {
	a, err := acct.DB.GetServerByName(
		acct.DB.NewSession(),
		event.Cmd.Msg["name"],
	)
	if err != nil {
		logrus.WithError(err).Warn("Cant get server")
		return
	}

	acct.answer(
		event.Client,
		event.Cmd.PayloadID,
		ansNuLoginPersona{
			Txn:       acctNuLoginPersona,
			ProfileID: a.ID,
			UserID:    a.ID,
			LobbyKey:  event.Client.PlayerData.LobbyKey,
		},
	)
}
