package fsys

import (
	"time"

	"gitlab.com/oiacow/goheroes/backend/config"
	"gitlab.com/oiacow/goheroes/backend/core"
	"gitlab.com/oiacow/goheroes/backend/core/codec"
)

// reqHello is definition of the fsys.Hello
// Packet ID: [192 0 0 1]
type reqHello struct {
	TXN             string `fesl:"TXN"`
	ClientString    string `fesl:"clientString"`
	Sku             int    `fesl:"sku"`
	Locale          string `fesl:"locale"`
	ClientPlatform  string `fesl:"clientPlatform"`
	ClientVersion   string `fesl:"clientVersion"`
	SdkVersion      string `fesl:"SDKVersion"`
	ProtocolVersion string `fesl:"protocolVersion"`
	FragmentSize    int    `fesl:"fragmentSize"`
	ClientType      string `fesl:"clientType"`
}

type ansHello struct {
	// TXN stands for Taxon, sub-query name of the cmd.
	// Should be equal: Hello.
	TXN string `fesl:"TXN"`

	ConnTTL     int    `fesl:"activityTimeoutSecs"`
	ConnectedAt string `fesl:"curTime"`

	TheaterIP   string `fesl:"theaterIp"`
	TheaterPort int    `fesl:"theaterPort"`

	Domain           domainPartition `fesl:"domainPartition"`
	AddressRemapping string          `fesl:"addressRemapping,omitempty"`

	MessengerIP   string `fesl:"messengerIp"`
	MessengerPort int    `fesl:"messengerPort"`
}

type domainPartition struct {
	Name    string `fesl:"domain"`
	SubName string `fesl:"subDomain"`
}

// Hello handle fsys.Hello
// the first packet send by gameclient/game-server
func (fsys *ConnectSystem) Hello(client *core.Client, cmd *codec.Cmd) {
	client.Type = cmd.Msg["clientType"]

	ans := ansHello{
		TXN:           fsysHello,
		ConnTTL:       int((60 * time.Second).Seconds()),
		ConnectedAt:   time.Now().Format("Jan-02-2006 15:04:05 MST"),
		TheaterIP:     config.General.ThtrAddr,
		MessengerIP:   config.General.TelemetryIP,
		MessengerPort: config.General.TelemetryPort,
	}

	if fsys.ServerMode {
		ans.Domain = domainPartition{"eagames", "bfwest-server"}
		ans.TheaterPort = config.General.ThtrServerPort
	} else {
		ans.Domain = domainPartition{"eagames", "bfwest-dedicated"}
		ans.TheaterPort = config.General.ThtrClientPort
	}

	// ans.ErrorCode = (non zero) -> Hello exit with fsys.Goodbye

	fsys.answer(client, 0xC0000001, ans)
}
