package pnow

import (
	"github.com/sirupsen/logrus"
	"gitlab.com/oiacow/goheroes/backend/core"
)

const (
	sessionStateNEW       = "NEW"
	sessionStateDeleted   = "DELETED"
	sessionStateCancelled = "CANCELLED"
	sessionStateError     = "ERROR"
	sessionStateComplete  = "COMPLETE"
	sessionStateActive    = "ACTIVE"
)

const (
	resultTypeJoin = "JOIN"
)

type ansStatus struct {
	Txn          string                 `fesl:"TXN"`
	ID           statusPartition        `fesl:"id"`
	SessionState string                 `fesl:"sessionState"`
	Properties   map[string]interface{} `fesl:"props"`
}

type statusPartition struct {
	ID        int    `fesl:"id"`
	Partition string `fesl:"partition"`
}

type statusGame struct {
	LobbyID   int `fesl:"lid"`
	FitFactor int `fesl:"fit"`
	GameID    int `fesl:"gid"`
}

// Status handles pnow.Status cmd
func (pnow *PlayNow) Status(event core.EClientCmd) {
	logrus.Println("--pnow.Status--")
	game, err := pnow.MM.FindGame()
	if err != nil {
		logrus.WithError(err).Warn("Cant list available games in pnow.Status")
		// Return an error to client about failed matchmaking
		pnow.answer(event.Client, 0x80000000, ansStatus{
			Txn:          pnowStatus,
			ID:           statusPartition{1, event.Cmd.Msg["partition.partition"]},
			SessionState: sessionStateError,
		})
		return
	}

	games := []statusGame{
		{
			LobbyID:   1,
			GameID:    game.ID,
			FitFactor: 800,
		},
	}

	pnow.answer(
		event.Client,
		0x80000000, // event.Cmd.PayloadID,
		ansStatus{
			Txn:          pnowStatus,
			ID:           statusPartition{1, event.Cmd.Msg["partition.partition"]},
			SessionState: sessionStateComplete,
			Properties: map[string]interface{}{
				"resultType": resultTypeJoin,
				"games":      games,
			},
		},
	)
}
