package rank

import (
	"github.com/sirupsen/logrus"

	"gitlab.com/oiacow/goheroes/backend/core"
)

type reqGetStats struct {
	Owner      string   `fesl:"owner"`      // owner=5
	OwnerType  string   `fesl:"ownerType"`  // ownerType=1
	PeriodID   string   `fesl:"periodId"`   // periodId=0
	PeriodPast string   `fesl:"periodPast"` // periodPast=0
	Keys       []string `fesl:"keys"`       // keys.0=c_apr, keys.1=level ...
}

type ansGetStats struct {
	Txn       string      `fesl:"TXN"`
	OwnerID   int         `fesl:"ownerId"`
	OwnerType int         `fesl:"ownerType"`
	Stats     []statsPair `fesl:"stats"`
}

type statsPair struct {
	Key   string `fesl:"key"`
	Text  string `fesl:"text,omitempty"`
	Value string `fesl:"value"`
}

// GetStats - Get basic stats about a soldier/owner (account holder)
func (r *Ranking) GetStats(event core.EClientCmd) {
	switch event.Client.GetClientType() {
	case "server":
		r.serverGetStats(&event)
	default:
		r.clientGetStats(&event)
	}
}

func (r *Ranking) serverGetStats(event *core.EClientCmd) {
	r.getStats(event)
}

func (r *Ranking) clientGetStats(event *core.EClientCmd) {
	r.getStats(event)
}

func (r *Ranking) getStats(event *core.EClientCmd) {
	if event.Cmd.Msg["owner"] == "Current" {
		// In the tutorial "Current" is reserved name for the hero.
		return
	}

	owner, err := event.Cmd.Msg.IntVal("owner")
	if err != nil {
		logrus.
			WithField("owner", event.Cmd.Msg["owner"]).
			WithField("cmd", "rank.GetStats").
			Warnf("Cant parse owner")
		return
	}

	heroID := event.Client.PlayerData.HeroID
	if heroID == 0 {
		if event.Client.Type == "server" {
			heroID = owner
		} else {
			player, err := r.DB.GetPlayerByID(r.DB.NewSession(), owner)
			if err != nil {
				logrus.Println("error selecting selectplayerbyid")
				return
			}
			if player.SelectedHero.Valid == false {
				logrus.Println("error selecting line 75")
				return
			}

			logrus.Warnf("GetStats (client), replacing owner with heroID with %v", player.SelectedHero.Int64)
			heroID = int(player.SelectedHero.Int64)
		}
	}

	logrus.
		WithField("heroID", heroID).
		WithField("owner", owner).
		WithField("clientType", event.Client.Type).
		Print("Requesting rank.GetStats")

	heroStats, err := r.DB.GetHeroStats(r.DB.NewSession(), heroID)
	if err != nil {
		logrus.WithError(err).Errorf("Cant retrieve stats of hero %d", heroID)
		return
	}

	keys := event.Cmd.Msg.ArrayStrings("keys")
	stats, err := r.fetchStats(&heroStats, keys)
	if err != nil {
		logrus.WithError(err).Error("Cant retrieve given stats")
		return
	}

	r.answer(
		event.Client,
		event.Cmd.PayloadID,
		ansGetStats{
			Txn:       rankGetStats,
			OwnerID:   heroID,
			OwnerType: 1,
			Stats:     stats,
		},
	)
}
