package rank

import (
	"fmt"

	"github.com/sirupsen/logrus"

	"gitlab.com/oiacow/goheroes/backend/core"
)

type reqGetStatsForOwners struct {
	Txn      string       `fesl:"TXN"`  // =GetStatsForOwners
	Keys     []string     `fesl:"keys"` // = elo,c_team,xp,level ...
	OwnerID  []statsOwner `fesl:"owners"`
	PeriodID int          `fesl:"periodId"` // =0
}

type statsOwner struct {
	OwnerID   int `fesl:"ownerId"`   // ID of hero
	OwnerType int `fesl:"ownerType"` // =1
}

type ansGetStatsForOwners struct {
	Txn   string           `fesl:"TXN"`
	Stats []statsContainer `fesl:"stats"`
}

type statsContainer struct {
	Stats     []statsPair `fesl:"stats"`
	OwnerID   int         `fesl:"ownerId"`
	OwnerType int         `fesl:"ownerType"`
}

func (r *Ranking) GetStatsForOwners(event core.EClientCmd) {
	switch event.Client.GetClientType() {
	case "server":
		r.serverGetStatsForOwners(&event)
	default:
		r.clientGetStatsForOwners(&event)
	}
}

func (r *Ranking) serverGetStatsForOwners(event *core.EClientCmd) {
	r.getStatsForOwners(event)
}

func (r *Ranking) clientGetStatsForOwners(event *core.EClientCmd) {
	r.getStatsForOwners(event)
}

func (r *Ranking) getStatsForOwners(event *core.EClientCmd) {
	stats := []statsContainer{}

	owners, err := event.Cmd.Msg.IntVal("owners.[]")
	if err != nil {
		logrus.WithError(err).Warn("GetStatsForOwners")
		return
	}

	keys := event.Cmd.Msg.ArrayStrings("keys")
	for i := 0; i < owners; i++ {
		owner, err := event.Cmd.Msg.IntVal(fmt.Sprintf("owners.%d.ownerId", i))
		if err != nil {
			logrus.WithError(err).Warn("Cant parse onwer in GetStatsForOwners")
		}

		heroStats, err := r.DB.GetHeroStats(r.DB.NewSession(), owner)
		if err != nil {
			logrus.WithError(err).Errorf("Cant retrieve stats of hero %d", owner)
			return
		}

		statsPairs, err := r.fetchStats(&heroStats, keys)
		if err != nil {
			logrus.WithError(err).Warn("rank.GetStatsForOwners: Cant fetch stats for hero of ID %d", owner)
			return
		}

		stats = append(stats, statsContainer{
			OwnerID:   owner,
			OwnerType: 1,
			Stats:     statsPairs,
		})
	}

	r.answer(event.Client, event.Cmd.PayloadID, ansGetStatsForOwners{
		Txn:   rankGetStatsForOwners,
		Stats: stats,
	})
}
